//
// Copyright (C) 2008-2010 Reto Gantenbein
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//

#ifndef VIFBACKEND_H
#define VIFBACKEND_H

#include <omnetpp.h>
#include <platdep/sockets.h>
#include <arpa/inet.h>				// for address conversion
#include <netdb.h>					// for ipv6 stuff
#include <PhyControlInfo_m.h>		// for setting radio channel
#include "EmulationRTScheduler.h"
#include "RAWEtherFrame.h"
#include "PMTypes.h"


class VifBackend : public cSimpleModule
{
  private:
	cModule *host;						// pointer to VirtualHost module
	cModule *vif[MAX_NUM_VIF];			// pointers to Ieee80211NicAdhoc modules
	struct addrinfo addrInfo;			// address information
	struct sockaddr_storage hostAddr;	// host IP/port
	int numVif;							// number of virtual interfaces

	void sendToHost(char *msg, int length);

  protected:
    virtual void initialize();
    virtual void handleMessage(cMessage *msg);
    void setRTS(int8_t index, int16_t rts);
    void setRetry(int8_t index, int16_t retry);
    void setTxPower(int8_t index, int16_t txpower);
    void setSensitivity(int8_t index, int16_t sensitivity);
    void setChannel(int8_t index, int16_t channel);

  public:
	VifBackend();
	virtual ~VifBackend();
    void sendAck(unsigned int msgId);
    void sendToModel(char *wlanPkt, int length);
    void updateVifParameter(configNotification *info);
    void initializeVifParameters(vifRegistrationData vifReg[]);
};

#endif  // VIFBACKEND_H
